@@ -5,7 +5,7 @@ Use pytest to run this test suite.
 The tests are executed in external processes: setproctitle should
 never be imported directly from here.
 
-Copyright (c) 2009-2020 Daniele Varrazzo <daniele.varrazzo@gmail.com>
+Copyright (c) 2009-2021 Daniele Varrazzo <daniele.varrazzo@gmail.com>
 """
 
 import os
@@ -16,13 +16,10 @@ import subprocess as sp
 
 import pytest
 
-from .conftest import run_script
+from .conftest import run_script, skip_if_no_proc_cmdline, skip_if_no_proc_env
+from .conftest import skip_if_macos, skip_if_pypy, skip_if_win32
 
-IS_PYPY = "__pypy__" in sys.builtin_module_names
-
-if sys.platform == 'win32':
-    pytest.skip("skipping Posix tests on Windows",
-                allow_module_level=True)
+pytestmark = [skip_if_win32]
 
 
 def test_runner():
@@ -35,30 +32,10 @@ print(10 + 20)
     assert rv == "30\n"
 
 
-def test_no_import_side_effect():
-    """
-    Check that importing the module doesn't cause side effects.
-    """
-    rv = run_script(
-        """
-import os
-
-def print_stuff():
-    for fn in "cmdline status comm".split():
-        if os.path.exists(f"/proc/self/{fn}"):
-            with open(f"/proc/self/{fn}") as f:
-                print(f.readline().rstrip())
-
-print_stuff()
-print("---")
-import setproctitle
-print_stuff()
-"""
-    )
-    before, after = rv.split("---\n")
-    assert before == after
-
-
+@pytest.mark.skipif(
+    'sys.platform == "darwin" and os.environ.get("CIBW_TEST_COMMAND")',
+    reason="f*cked up binary name",
+)
 def test_init_getproctitle():
     """getproctitle() returns a sensible value at initial call."""
     rv = run_script(
@@ -92,6 +69,22 @@ print(os.popen("ps -x -o pid,command 2> /dev/null").read())
     assert title == "Hello, world!"
 
 
+@pytest.mark.skipif('sys.platform != "darwin"', reason="Mac only test")
+def test_setproctitle_darwin():
+    """Mac Activity monitor shows correct info"""
+    rv = run_script(
+        r"""
+import setproctitle
+setproctitle.setproctitle('QwErTyZxCvB')
+
+import os
+print(os.popen("lsappinfo find name=QwErTyZxCvB 2> /dev/null").read())
+"""
+    )
+    m = re.search(r'''ASN:.*"QwErTyZxCvB"''', rv)
+    assert m
+
+
 def test_prctl():
     """Check that prctl is called on supported platforms."""
     linux_version = []
@@ -316,7 +309,7 @@ def test_weird_path(tmp_path, spt_directory):
     except UnicodeEncodeError:
         pytest.skip("file system doesn't support unicode")
 
-    exc = dir / "python"
+    exc = dir / os.path.basename(sys.executable)
     os.symlink(sys.executable, exc)
 
     rv = run_script(
@@ -343,13 +336,11 @@ print(os.popen("ps -x -o pid,command 2> /dev/null").read())
 
 
 @pytest.mark.embedded
-@pytest.mark.skipif(IS_PYPY, reason="skip test, pypy")
+@skip_if_pypy
+@skip_if_macos
+@skip_if_no_proc_cmdline
 def test_embedded(pyrun, spt_directory):
-    """Check the module works with embedded Python.
-    """
-    if not os.path.exists("/proc/%s/cmdline" % os.getpid()):
-        pytest.skip("known failure: '/proc/PID/cmdline' not available")
-
+    """Check the module works with embedded Python."""
     rv = run_script(
         f"""
 import sys
@@ -372,12 +363,11 @@ print(os.popen("ps -x -o pid,command 2> /dev/null").read())
 
 
 @pytest.mark.embedded
-@pytest.mark.skipif(IS_PYPY, reason="skip test, pypy")
+@skip_if_pypy
+@skip_if_macos
+@skip_if_no_proc_cmdline
 def test_embedded_many_args(pyrun, spt_directory):
     """Check more complex cmdlines are handled in embedded env too."""
-    if not os.path.exists("/proc/%s/cmdline" % os.getpid()):
-        pytest.skip("known failure: '/proc/PID/cmdline' not available")
-
     rv = run_script(
         f"""
 import sys
@@ -400,11 +390,9 @@ print(os.popen("ps -x -o pid,command 2> /dev/null").read())
     assert title == "Hello, embedded!"
 
 
+@skip_if_no_proc_env
 def test_noenv():
     """Check that SPT_NOENV avoids clobbering environ."""
-    if not os.path.exists("/proc/self/environ"):
-        pytest.skip("'/proc/self/environ' not available")
-
     env = os.environ.copy()
     env["SPT_TESTENV"] = "testenv"
     rv = run_script(
@@ -439,9 +427,9 @@ print('SPT_TESTENV=testenv' in open('/proc/self/environ').read())
     ), "title (len {title_len}) not limited to argv (len {cmdline_len})"
 
 
+@skip_if_no_proc_env
 def test_large_env(monkeypatch):
-    """Check that large environment doesn't get clobbered.
-    """
+    """Check that large environment doesn't get clobbered."""
     monkeypatch.setenv("SPT_NOENV", "1")
     for c in string.ascii_uppercase:
         monkeypatch.setenv(
@@ -475,21 +463,40 @@ with open("/proc/self/environ", "rb") as f:
 
 
 def test_clear_segfault():
-    rv = run_script(
+    run_script(
         r"""\
 import os
 from setproctitle import setproctitle
 os.environ.clear()
 setproctitle("Test")
-""")
+"""
+    )
+
+
+def test_fork_segfault():
+    run_script(
+        """\
+import multiprocessing as mp
+from setproctitle import setproctitle
+
+def foo():
+    setproctitle('title in child')
+
+setproctitle('title in parent')
+mp.set_start_method("fork")
+p = mp.Process(target=foo)
+p.start()
+p.join()
+assert p.exitcode == 0, f"p.exitcode is {p.exitcode}"
+"""
+    )
 
 
 # Support functions
 
 
 def _clean_up_title(title):
-    """Clean up a string from the prefix added by the platform.
-    """
+    """Clean up a string from the prefix added by the platform."""
     # BSD's setproctitle decorates the title with the process name.
     if "bsd" in sys.platform:
         procname = os.path.basename(sys.executable)
